<?php

function customvote_fivestar_info() {
  $info = array(
    'fivestar' => array(
      'name' => 'Five Stars',
      'summary_func' => 'average',
      'description'=>'Five Star vote sets allow users to vote using an Amazon-style five star system. There are no configuration options for this vote set type.',
      )
    );
  return $info;
}

function customvote_get_fivestar_widget_form(&$form) {
  $settings = $form['meta']['settings']['#value'];
  if ($settings->average) {
    $form['average'] = array(
      '#type' => 'hidden',
      '#value' => TRUE,
    );
  }

  foreach($form['meta']['criteria']['#value'] as $key=>$c) {
    $form['criteria'][$key] = array(
      '#type' => 'select',
      '#title' => $c,
      '#default_value' => $form['meta']['existing_votes']['#value'][$key]->value,
      '#options' => array(0=>"", 20=>"*", 40=>"* *", 60=>"* * *", 80=>"* * * *", 100=>"* * * * *"),
    );
  }
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => $form['meta']['settings']['#value']->caption,
  );
}


function theme_customvote_fivestar_widget_form($form) {
  foreach (element_children($form['criteria']) as $tag) {
    $row = array();
    $row[] = '<strong>'.$form['criteria'][$tag]['#title'].'</strong>';
    unset($form['criteria'][$tag]['#title']);
    $row[] = form_render($form['criteria'][$tag]);
    $rows[] = $row;
  }
  $row = array();
  $row[] = '';
  $row[] = form_render($form['submit']);
  $rows[] = $row;

  $output = theme('table', NULL, $rows, array('class' => 'votewidget fivestar'));
  $output .= form_render($form);
  return $output;
}


function theme_customvote_fivestar_results($settings, $content_type, $content_id) {
  $average = votingapi_get_voting_result($content_type, $content_id, VOTINGAPI_VALUE_TYPE_PERCENT, VOTINGAPI_VALUE_DEFAULT_TAG, 'average');
  
  $output = '<p class="voteresult">Current score: ' . customvote_fivestar_format_value($average) . '<br />';
  
  return $output;
}

function customvote_fivestar_widget_form_submit($form_id, $form_values) {
  // set up the basic variables.

  foreach (element_children($form_values['criteria']) as $tag) {
    $value = $form_values['criteria'][$tag];
    if ($value > 100) {
      $value = 100;
    }
    if ($value < 0) {
      $value = 0;
    }
    $total += $value;
    $count++;

    $vote_object->tag = $tag;
    $vote_object->value_type = VOTINGAPI_VALUE_TYPE_PERCENT;
    $vote_object->value = $form_values['criteria'][$tag];
    $vote_objects[] = $vote_object;
    unset($vote_object);
  }
  
  // If necessary, calculate an ovreall vote value for all criteria
  if ($form_values['average'] && $count > 1) {
    $vote_object->tag = VOTINGAPI_VALUE_DEFAULT_TAG;
    $vote_object->value_type = VOTINGAPI_VALUE_TYPE_PERCENT;
    $vote_object->value = $total / $count;
    $vote_objects[] = $vote_object;
  }
  votingapi_set_vote($form_values['content_type'], $form_values['content_id'], $vote_objects);
}


function customvote_fivestar_format_value($vobj) {
  return theme_customvote_fivestar_rating($vobj->value);
}

function theme_customvote_fivestar_rating($pct) {
  $score = round($pct / 20, 1);
  $stars = round($score);
  
  $path = '/' . drupal_get_path('module', 'customvote') . '/vote_types/fivestar/';
  for ($i = 1; $i < 6; $i++) {
    if ($i <= $score) {
      $output .= "<img src='" . $path . "star_on.png' />";
    }
    else {
      if ($i <= $stars) {
        $output .= "<img src='" . $path . "star_half.png' />";
      }
      else {
        $output .= "<img src='" . $path . "star_off.png' />";
      }
    }
  }
  $output .= " ($score)";
  
  return $output;
}