<?php

function customvote_named_info() {
  $info = array(
    'named' => array(
      'name' => 'Named votes',
      'summary_func' => 'sum',
      'description'=>'Named value vote sets contain 1-n votes, each with a numerical value. Users cast votes by name, and the points are totaled to determine the content\'s rating.',
      )
    );
  return $info;
}

function customvote_get_named_widget_form(&$form) {
  $votes = _customvote_get_votes_for_set($form['sid']['#value']);
  $options = array();
  foreach ($votes as $vid=>$vote) {
    $options[$vid] = $vote->vote;
  }
  foreach($form['meta']['criteria']['#value'] as $key=>$c) {
    $form['criteria'][$key] = array(
      '#type' => 'select',
      '#title' => $c,
      '#options' => $options,
      '#default_value' => $form['meta']['existing_votes']['#value'][$key]->value,
    );
  }
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => $form['meta']['settings']['#value']->caption,
  );
}

function theme_customvote_named_widget_form($form) {
  foreach (element_children($form['criteria']) as $tag) {
    $row = array();
    $row[] = '<strong>'.$form['criteria'][$tag]['#title'].'</strong>';
    unset($form['criteria'][$tag]['#title']);
    $row[] = form_render($form['criteria'][$tag]);
    $rows[] = $row;
  }
  $row = array();
  $row[] = '';
  $row[] = form_render($form['submit']);
  $rows[] = $row;

  $output = theme('table', NULL, $rows, array('class' => 'votewidget named'));
  $output .= form_render($form);
  return $output;
}

function theme_customvote_named_results($settings, $content_type, $content_id) {
  $sum = votingapi_get_voting_result($content_type, $content_id, VOTINGAPI_VALUE_TYPE_TOKEN, VOTINGAPI_VALUE_DEFAULT_TAG, 'sum');
  $count = votingapi_get_voting_result($content_type, $content_id, VOTINGAPI_VALUE_TYPE_TOKEN, VOTINGAPI_VALUE_DEFAULT_TAG, 'count');
  
  $output = '<p class="voteresult">Current score: ' . customvote_named_format_value($sum) . '<br />';
  $output .= '(' . $count->value . ' users have voted)</p>';
  
  return $output;
}

function customvote_named_widget_form_submit($form_id, $form_values) {
  // set up the basic variables.
  $sid = $form_values['sid'];
  $vote_objects = array();

  foreach (element_children($form_values['criteria']) as $tag) {
    $vote_object->tag = $tag;
    $vote_object->value_type = VOTINGAPI_VALUE_TYPE_KEY;
    $vote_object->value = $form_values['criteria'][$tag];
    $vote_objects[] = $vote_object;
    unset($vote_object);
  }
  votingapi_set_vote($form_values['content_type'], $form_values['content_id'], $vote_objects);
}



/*******************************************/
function customvote_named_set_edit($set) {
  $sid = $set->sid;
  $result = db_query('SELECT vid, sid, vote, weight, value FROM {customvote_votes} WHERE sid = %d ORDER BY weight', $set->sid);

  $form['sid'] = array(
    '#type' => 'hidden',
    '#value' => $sid,
  );

  while ($vote = db_fetch_object($result)) {
    $form['old'][$vote->vid]['vote'] = array(
      '#type' => 'textfield',
      '#default_value' => $vote->vote,
      '#size' => 32,
      '#maxlength' => 64,
    );

    $form['old'][$vote->vid]['value'] = array(
      '#type' => 'textfield',
      '#default_value' => $vote->value,
      '#size' => 3,
      '#maxlength' => 6,
    );

    $form['old'][$vote->vid]['weight'] = array(
      '#type' => 'weight',
      '#default_value' => $vote->weight,
      '#delta' => 10,
    );

    $form['old'][$vote->vid]['delete'] = array(
      '#type' => 'checkbox',
      '#default_value' => 0,
      '#return_value' => 1,
    );
  }
  
  // Add a set of fields for new votes.
  $form['new']['vote'] = array(
   '#type' => 'textfield',
   '#size' => 32,
   '#maxlength' => 64,
  );

  $form['new']['value'] = array(
   '#type' => 'textfield',
   '#size' => 3,
   '#maxlength' => 6,
  );

  $form['new']['weight'] = array(
    '#type' => 'weight',
    '#delta' => 10,
  );

  $form['#tree'] = TRUE; // make sure we preserve all hierarchial values.
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save changes'),
  );
  $output .= drupal_get_form('customvote_named_set_edit_form', $form);
  return $output;
}

function customvote_named_set_edit_form_submit($form_id, $form_values) {
  // loop through the fields for old votes, update or delete as necessary.
  foreach (element_children($form_values['old']) as $vid) {
    if ($form_values['old'][$vid]['delete'] == 1) {
      db_query('DELETE FROM {customvote_votes} WHERE vid = %d', $vid);
    }
    else {
      db_query("UPDATE {customvote_votes} SET vote = '%s', value = %d, weight = %d WHERE vid = %d", $form_values['old'][$vid]['vote'], $form_values['old'][$vid]['value'], $form_values['old'][$vid]['weight'], $vid);
    }
  }  
  
  // if a new vote has been added, insert it.
  if ($form_values['new']['vote'] != '' && $form_values['new']['value'] != '') {
    db_query("INSERT INTO {customvote_votes} (vid, sid, vote, value, weight) VALUES (%d, %d, '%s', %d, %d)", db_next_id('customvote_votes'), $form_values['sid'], $form_values['new']['vote'], $form_values['new']['value'], $form_values['new']['weight']);
  }

  drupal_set_message(t('the changes have been saved.'));
  drupal_goto('admin/vote/sets/edit/' . $form_values['sid']); // do this because we want the form completely rebuilt.
}

function theme_customvote_named_set_edit_form($form) {
  $header = array(t('vote'), t('value'), t('weight'), t('delete'));
  
  // render the existing-vote fields.
  foreach (element_children($form['old']) as $vid) {
    $row = array();
    $row[] = form_render($form['old'][$vid]['vote']);
    $row[] = form_render($form['old'][$vid]['value']);
    $row[] = form_render($form['old'][$vid]['weight']);
    $row[] = form_render($form['old'][$vid]['delete']);
    $rows[] = $row;
  }
  
  if (!$rows) {
    $rows[] = array(array('data' => t('No votes have been defined.'), 'colspan' => '4'));
  }
  
  // render the add-a-vote fields.
  $row = array();
  $row[] = form_render($form['new']['vote']);
  $row[] = form_render($form['new']['value']);
  $row[] = form_render($form['new']['weight']);
  $row[] = '';
  $rows[] = $row;
  
  $output .= theme('table', $header, $rows);
  $output .= form_render($form);
  return $output;
}


function customvote_named_calc_function($settings, &$cache, $votes, $content_type, $content_id) {
  $uids = array();
  foreach($votes as $vote) {

    // we can't do PERFECT matching, really -- but we know that if we get this far, 
    // we've been told to handle voting for this particular content type.
    // this only causes problems, really, when people have set up multiple
    // overlapping vote systems that use a lot of the same parameters. That's
    // a problem to solve another day.

    // the right value type.
    if ($vote->value_type != VOTINGAPI_VALUE_TYPE_KEY) {
      continue;
    }

    // if there's a defined list of criteria, and the current vote's tag isn't in it, bail out. we shouldn't be calculating it.
    if (is_array($settings->criteria) && !in_array($vote->tag, array_keys($settings->criteria))) {
      continue;
    }
    
    // By default, this gets set by the api.
    // It's worthless for us and will only clutter up the DB.
    unset($cache[$vote->tag][$vote->value_type][$vote->value]);
    
    // this is slightly voodoo-esque. We're taking in votes with a 'key' value type
    // and writing out result records with a 'token' value type. it's voodoo, but it works.
    $value = _customvote_get_named_value($vote->value);
    $cache[$vote->tag][VOTINGAPI_VALUE_TYPE_TOKEN]['total'] += $value;

    $total += $value;
    if (!in_array($vote->uid, $uids)) {
      $uids[] = $vote->uid;
      $count += 1;
    }
  }
  if ($settings->average && is_array($settings->criteria) && count($settings->criteria) > 0) {
    $cache[VOTINGAPI_VALUE_DEFAULT_TAG][VOTINGAPI_VALUE_TYPE_TOKEN]['total'] = $total;
  }
  $cache[VOTINGAPI_VALUE_DEFAULT_TAG][VOTINGAPI_VALUE_TYPE_TOKEN]['count'] = $count;
}




// if you pass in an $account object, it must have $uid and $roles elements
function _customvote_get_named_value($vid) {
  $sql = 'SELECT value FROM {customvote_votes} WHERE vid = %d';
  return db_result(db_query_range($sql, $vid, 0, 1)); // see comment_moderate() for analogous function
}


function customvote_named_format_value($vobj) {
  if (isset($vobj->uid)) {
    // it's an individual vote. Get the value name.
    $vote = db_result(db_query('SELECT vote FROM {customvote_votes} WHERE vid = %d', $vobj->value));
    return $vote;
  }
  else {
    // It's a result. Format as points.
    return $vobj->value . ' points';
  }
}